@@ -11,10 +11,9 @@ mxl_pref <- logitr(
   data     = yogurt,
   outcome  = 'choice',
   obsID    = 'obsID',
+  panelID  = 'id',
   pars     = c('price', 'feat', 'brand'),
   randPars = c(feat = 'n', brand = 'n'),
-  # You should run a multistart for MXL models since they are non-convex,
-  # but it can take a long time. Here I just use 5 starts for brevity:
   numMultiStarts = 10
 )
 
@@ -26,21 +25,31 @@ mxl_wtp <- logitr(
   data       = yogurt,
   outcome    = 'choice',
   obsID      = 'obsID',
+  panelID    = 'id',
   pars       = c('feat', 'brand'),
   price      = 'price',
   randPars   = c(feat = 'n', brand = 'n'),
   modelSpace = 'wtp',
-  # You should run a multistart for MXL models since they are non-convex,
-  # but it can take a long time. Here I just use 5 starts for brevity:
   numMultiStarts = 10,
-  # Use the computed WTP from the preference space model as the starting
-  # values for the first run:
   startVals = wtp_mxl_pref$Estimate
 )
 
 # Compare results
 wtpCompare(mxl_pref, mxl_wtp, "price")
 
+# Multistart MXL model in the Preference Space with correlated heterogeneity
+mxl_pref_cor <- logitr(
+  data     = yogurt,
+  outcome  = 'choice',
+  obsID    = 'obsID',
+  panelID  = 'id',
+  pars     = c('price', 'feat', 'brand'),
+  randPars = c(feat = 'n', brand = 'n'),
+  numMultiStarts = 10,
+  correlation = TRUE
+)
+
 # Save results
 saveRDS(mxl_pref, here::here('inst', 'extdata', 'mxl_pref.Rds'))
 saveRDS(mxl_wtp,  here::here('inst', 'extdata', 'mxl_wtp.Rds'))
+saveRDS(mxl_pref_cor, here::here('inst', 'extdata', 'mxl_pref_cor.Rds'))
